# -*- coding: utf-8 -*-
#
# gistore -- Backup files using DVCS, such as git.
# Copyright (C) 2010 Jiang Xin <jiangxin@ossxp.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

import os
from signal import SIGINT
from gistore.config import *
from gistore.errors import *
import logging

log = logging.getLogger('gist.utils')


def warn_if_error(proc, cmdline=""):
    if isinstance(cmdline, (list,tuple)):
        cmdline = " ".join(cmdline)

    # use proc.communicate() instead of proc.wait() or read buffer before call proc.wait(),
    # otherwize if buffer overflow, process will hang!
    output, error_output = proc.communicate()
    if error_output:
        output = output and output + "\n" + error_output or error_output

    if proc.returncode != 0:
        log.warning("Last command: %s\n\tgenerate warnings with returncode %d." % (cmdline, proc.returncode))
        if output:
            log.warning( "Command output:\n" + output )
    else:
        log.debug( "command: %s" % cmdline )
        if output:
            log.debug( "output:\n" + output )


def exception_if_error(proc, cmdline="", outtest=None):
    if isinstance(cmdline, (list,tuple)):
        cmdline = " ".join(cmdline)

    # use proc.communicate() instead of proc.wait() or read buffer before call proc.wait(),
    # otherwize if buffer overflow, process will hang!
    output, error_output = proc.communicate()
    if error_output:
        output = output and output + "\n" + error_output or error_output

    success = False
    if outtest is not None:
        for line in output.splitlines():
            if not success and outtest(line):
                success = True
                log.debug( "Command not failed, matched line found: %s" % line )
                break

    if not success and proc.returncode != 0:
        msg = "Last command: %s\n\tgenerate ERRORS with returncode %d!" % (cmdline, proc.returncode)
        log.critical( msg )
        if output:
            log.critical( "Command output:\n" + output )
        raise CommandError( msg )
    else:
        log.debug("Command: %s" % cmdline)
        if output:
            log.debug( "Command output:\n" + output )


def get_exception(e):
    traceback = True
    if '__module__' in dir(e) and e.__module__.startswith("gistore"):
        traceback = False
    return "Exception caught abort: %s" % exception_to_unicode(e, traceback=traceback)


# exception_to_unicode and to_unicode is borrowed from Trac.
def exception_to_unicode(e, traceback=""):
    message = '%s: %s' % (e.__class__.__name__, to_unicode(e))
    if traceback:
        import traceback
        from StringIO import StringIO
        tb = StringIO()
        traceback.print_exc(file=tb)
        traceback_only = tb.getvalue().split('\n')[:-2]
        message = '\n%s\n%s' % (to_unicode('\n'.join(traceback_only)), message)
    return message


def to_unicode(text, charset=None):
    """Convert a `str` object to an `unicode` object.

    If `charset` is given, we simply assume that encoding for the text,
    but we'll use the "replace" mode so that the decoding will always
    succeed.
    If `charset` is ''not'' specified, we'll make some guesses, first
    trying the UTF-8 encoding, then trying the locale preferred encoding,
    in "replace" mode. This differs from the `unicode` builtin, which
    by default uses the locale preferred encoding, in 'strict' mode,
    and is therefore prompt to raise `UnicodeDecodeError`s.

    Because of the "replace" mode, the original content might be altered.
    If this is not what is wanted, one could map the original byte content
    by using an encoding which maps each byte of the input to an unicode
    character, e.g. by doing `unicode(text, 'iso-8859-1')`.
    """
    if not isinstance(text, str):
        if isinstance(text, Exception):
            # two possibilities for storing unicode strings in exception data:
            try:
                # custom __str__ method on the exception (e.g. PermissionError)
                return unicode(text)
            except UnicodeError:
                # unicode arguments given to the exception (e.g. parse_date)
                return ' '.join([to_unicode(arg) for arg in text.args])
        return unicode(text)
    if charset:
        return unicode(text, charset, 'replace')
    else:
        try:
            return unicode(text, 'utf-8')
        except UnicodeError:
            return unicode(text, locale.getpreferredencoding(), 'replace')


# vim: et ts=4 sw=4



# -*- coding: utf-8 -*-
#
# gistore -- Backup files using DVCS, such as git.
# Copyright (C) 2010 Jiang Xin <jiangxin@ossxp.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

import os
import sys
import re
from subprocess import Popen, PIPE, STDOUT
from copy import deepcopy
import logging

from gistore.utils  import *
from gistore.config import *
from gistore.errors import *
from gistore        import versions

log = logging.getLogger('gist.api')

class Gistore(object):

    def __init__(self, task=None, create=False):
        self.root = None
        self.scm = None

        self.cmd_mount = ["mount", "--rbind"]
        self.cmd_umount = ["umount"]
        self.cmd_umount_force = ["umount", "-f", "-l"]

        # Users other than root, may use other painless command
        if os.getuid() != 0:
            self.cmd_umount_force.insert(0, "sudo")
            if os.system("which bindfs >/dev/null 2>&1") == 0:
                self.cmd_mount = ["bindfs"]
                self.cmd_umount = ["fusermount","-u"]
            elif os.system("which sudo >/dev/null 2>&1") == 0:
                self.cmd_mount.insert(0, "sudo")
                self.cmd_umount.insert(0, "sudo")

        self.__init_task(task, create)

    def __init_task(self, taskname, create):
        """Set default root dir according to task name. Task name can be:
          * Task name in /etc/gistore/tasks/, which is a symbol link to real
            gistore backup directory
          * Absolute dir name, such as: /backup/store1/
          * Relative dir name from current working directory
        """
        if not taskname:
            taskname = os.getcwd()
        elif (not taskname.startswith('/') and
              not taskname.startswith('./') and
              not taskname.startswith('../') and
              taskname != '.' and
              taskname != '..'):
            path = os.path.join(cfg.sys_config_dir, "tasks", taskname)
            if os.path.exists(path):
                taskname = path
            else:
                taskname = os.path.join(os.getcwd(), taskname)

        self.root = os.path.realpath(taskname)

        if create:
            if os.path.exists( self.root ):
                raise TaskAlreadyExistsError("Task already exists in: %s." % self.root)
            else:
                os.makedirs(self.root)
        elif not os.path.exists( os.path.join(self.root, GISTORE_CONFIG_DIR, "config") ):
                raise TaskNotExistsError("Task does not exists: %s." % os.path.join(self.root, GISTORE_CONFIG_DIR, "config") )

        # Create needed directories
        check_dirs = [ os.path.join( self.root, GISTORE_LOG_DIR ),
                       os.path.join( self.root, GISTORE_LOCK_DIR ),]
        for dir in check_dirs:
            if not os.path.exists( dir ):
                os.makedirs( dir )

        # Set file log
        filelog = logging.FileHandler( os.path.join( self.root, GISTORE_LOG_DIR, "gitstore.log" ) )
        filelog.setLevel( cfg.log_level )
        # set a format which is simpler for filelog use
        formatter = logging.Formatter('%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
        filelog.setFormatter(formatter)
        # add the handler to the root logger
        logging.getLogger('').addHandler(filelog)


        # Taskname is the abbr. link dir name in /etc/gistore/tasks/
        self.taskname = self.dir2task(self.root)

        # Initail self.store_list from .gistore/config file.
        self.repo_cfg = self.parse_config()
        old_version = self.repo_cfg["main.version"]

        # Upgrade config file if needed.
        if old_version is not None and old_version != versions.GISTORE_VERSION:
            self.upgrade(self.repo_cfg)

        # Scm backend initialized.
        scm = __import__("gistore.scm."+self.repo_cfg["main.backend"], globals(), {}, ["SCM"])
        work_tree = "/var/run/gistore/%s" % ( self.taskname or os.path.basename( self.root ) )
        work_tree = os.path.join( work_tree, str( os.getpid() ) )
        self.WORK_TREE = work_tree
        self.scm = scm.SCM(self.root, work_tree=self.WORK_TREE, backup_history=self.repo_cfg["main.backup_history"], backup_copies=self.repo_cfg["main.backup_copies"])

        # Upgrade scm if needed.
        if old_version is not None and old_version != versions.GISTORE_VERSION:
            self.scm.upgrade(old_version)

        # Check uid
        if os.getuid() != 0:
            if self.repo_cfg["main.root_only"]:
                raise PemissionDeniedError("Only root user allowed for task: %s" % (self.taskname or self.root))



    def init(self):
        self.scm.init()
        repo_cfg = {
                    "main.backend": cfg.backend,
                    "main.root_only": cfg.root_only,
                    "main.backup_history": cfg.backup_history,
                    "main.backup_copies": cfg.backup_copies,
                    "main.version": versions.GISTORE_VERSION,
                    'default.keep_empty_dir': False,
                    'default.keep_perm': False,
                   }
        self.save_config(repo_cfg)


    def upgrade(self, gistore_config):
        oldversion = gistore_config["main.version"]
        if oldversion == versions.GISTORE_VERSION:
            return
        
        if oldversion < 2:
            self.upgrade_2(gistore_config)

        self.save_config(gistore_config)


    def upgrade_2(self, gistore_config):
        gistore_config["main.version"] = 2


    def save_config(self, gistore_config={}):
        config_file = os.path.join(self.root, GISTORE_CONFIG_DIR, "config")
        if not os.path.exists(config_file):
            os.mkdir(os.path.dirname(config_file))

        main_buffer = []
        default_buffer = []
        for key, val in sorted(gistore_config.iteritems()):
            if key.startswith("default."):
                default_buffer.append( "%s = %s" % (
                         key[8:],
                         isinstance(val, bool) and ( val and "yes" or "no") or val,
                         ) )
            elif key.startswith("main."):
                main_buffer.append( "%s = %s" % (
                         key[5:],
                         isinstance(val, bool) and ( val and "yes" or "no") or val,
                         ) )

        store_buffer = []
        for path in sorted(self.store_list.keys()):
            if self.store_list[path].get('_system_', False):
                continue
            store_buffer.append( "[store %s]" % path )

            for key, val in self.store_list[path].iteritems():
                if gistore_config.get("default."+key, None) == val:
                    continue

                if isinstance(val, bool):
                    store_buffer.append( "%(key)s = %(val)s" % {
                            "key": key,
                            "val": val and "yes" or "no",
                            } )
                else:
                    store_buffer.append( key + " = " + val )
            store_buffer.append( "" )



        fp = open(config_file, "w")
        fp.write("""# Global config for all sections
[main]
%(main)s

[default]
%(default)s

# Define your backup list below. Section name begin with 'store ' will be backup.
# eg: [store /etc]
%(store)s
""" %       {
             'main':    "\n".join(main_buffer),
             'default': "\n".join(default_buffer),
             'store':   "\n".join(store_buffer),
            }
        )

        fp.close()


    def parse_config(self):
        """Initial self.store_list from cfg.store_list and .gistore/config file.
        """
        assert self.root

        repo_cfg = {
                    "main.backend": cfg.backend,
                    "main.root_only": cfg.root_only,
                    "main.backup_history": cfg.backup_history,
                    "main.backup_copies": cfg.backup_copies,
                    "main.version": None,
                    'default.keep_empty_dir': False,
                    'default.keep_perm': False,
                   }

        def update_config(config1, list2, prefix=""):
            config2 = {}
            if list2:
                for key, val in list2:
                    config2[prefix + key] = val
                    if val.lower() in ['0', 'f', 'false', 'n', 'no']:
                        config2[prefix + key] = False
                    elif val.lower() in ['1', 't', 'true', 'y', 'yes', 'ok']:
                        config2[prefix + key] = True
                config1.update(config2)

        def get_default(config1):
            config2 = {}
            for key, val in config1.iteritems():
                if not key.startswith("default."):
                    continue
                config2[key[8:]] = val
            return config2

        def validate_list(dir_list):
            """Remove duplicate dirs and dir which is already a git repository.
            """
            targets = []
            for p in sorted(map(os.path.realpath, dir_list)):
                if os.path.exists(p):
                    # Remove duplicate path
                    if len(targets) and ( targets[-1] == p or
                       p.startswith(os.path.join(targets[-1],'')) ):
                        log.warning("duplict path: %s" % p)
                        continue

                    # check if p is parent of self.root
                    elif self.root.startswith(os.path.join(p,"")) or self.root == p:
                        log.error("Not store root's parent dir: %s" % p)
                        continue

                    # check if p is child of self.root
                    elif p.startswith(os.path.join(self.root,"")):
                        if p != os.path.join(self.root, GISTORE_CONFIG_DIR):
                            log.error("Not store root's subdir: %s" % p)
                            continue

                    targets.append(p)

                else:
                    log.warning("%s not exists." % p)

            return targets

        self.store_list = {}
        dir_list = cfg.store_list.get(self.taskname, [])
        config_file = os.path.join(self.root, GISTORE_CONFIG_DIR, "config")
        if os.path.exists(config_file):
            # backup .gistore config files
            dir_list.append(os.path.join(self.root, GISTORE_CONFIG_DIR))
            from ConfigParser import ConfigParser
            cp=ConfigParser()
            cp.read(config_file)
            if cp.has_option('main', 'backend'):
                repo_cfg["main.backend"] = cp.get('main', 'backend')
            if cp.has_option('main', 'root_only'):
                repo_cfg["main.root_only"] = cp.getboolean('main', 'root_only')
            if cp.has_option('main', 'backup_history'):
                repo_cfg["main.backup_history"] = cp.getint('main', 'backup_history')
            if cp.has_option('main', 'backup_copies'):
                repo_cfg["main.backup_copies"] = cp.getint('main', 'backup_copies')
            if cp.has_option('main', 'version'):
                repo_cfg["main.version"] = cp.getint('main', 'version')
            else:
                # old version, needs to upgrade
                repo_cfg["main.version"] = 1

            if cp.has_section('default'):
                update_config(repo_cfg, cp.items('default'), "default.")

            for path in validate_list(dir_list):
                self.store_list[path] = get_default(repo_cfg)
                self.store_list[path]["_system_"] = True

            for section in filter(lambda n: n.startswith('store '), cp.sections()):
                self.store_list[section[6:].strip()] = get_default(repo_cfg)

            for section in filter(lambda n: n.startswith('store '), cp.sections()):
                path = os.path.realpath(section[6:].strip())
                if path in self.store_list.keys():
                    update_config(self.store_list[path], cp.items(section))

        else:
            for path in validate_list(dir_list):
                self.store_list[path] = get_default(repo_cfg)
                self.store_list[path]["_system_"] = True

        return repo_cfg

    def is_mount(self, src, dest):
        """Check whether src is mount on dest.
        If mount using bindfs, check by os.path.ismount is ok.
        If mount using mount --bind, check by src and dest's inode.
        """
        try:
            stat1 = os.stat(src)
            stat2 = os.stat(dest)
        except:
            return False
        return stat1.st_ino == stat2.st_ino or os.path.ismount(dest)

    def task2dir(self, task):
        return os.path.realpath(os.path.join(cfg.sys_config_dir, 'tasks', task))

    def dir2task(self, path):
        tasksdir = os.path.join(cfg.sys_config_dir, 'tasks')
        if os.path.exists(tasksdir):
            for t in os.listdir(tasksdir):
                if os.path.realpath(os.path.join(tasksdir, t)) == os.path.realpath(path):
                    return t
        return None

    def status(self):
        print "%18s : %s" % ("Task name", self.taskname and self.taskname or "-")
        print "%18s : %s" % ("Directory", self.root)
        print "%18s : %s" %  ("Backend", self.repo_cfg["main.backend"])
        print "%18s : %d commits * %d copies" %  ( "Backup capability", self.repo_cfg["main.backup_history"], self.repo_cfg["main.backup_copies"] )
        print "%18s :" % "Backup list"
        for k,v in self.store_list.iteritems():
            print "    %s (%s%s)" % (
                    k,
                    v.get("keep_perm") and "P" or "-",
                    v.get("keep_empty_dir") and "K" or "-")

    def __mnt_target(self, p):
        if p == os.path.join(self.root, GISTORE_CONFIG_DIR):
            return os.path.join( self.root, self.WORK_TREE, GISTORE_CONFIG_DIR.rstrip('/') )
        else:
            return os.path.join( self.root, self.WORK_TREE, p.lstrip('/'))

    def mount(self):
        self.lock("mount")

        # Create work_tree if not exists.
        if not os.path.exists( self.WORK_TREE ):
            try:
                os.makedirs( self.WORK_TREE, mode=0777 )
            except OSError:
                raise PemissionDeniedError("Can not create run-time dir: %s. You can override it in config file." % self.WORK_TREE)

        for p in self.store_list.keys():
            if os.path.isdir(p):
                target = self.__mnt_target(p)
                if not os.path.exists(target):
                    os.makedirs(target)
            elif os.path.isfile(p):
                target = self.__mnt_target(p)
                if not os.path.exists(os.path.dirname(target)):
                    os.makedirs(os.path.dirname(target))
                if not os.path.exists(target):
                    os.mknod(target, 0644)
            else:
                log.error("Mount failed. Unknown file type: %s." % p)
                continue

            if self.is_mount(p, target):
                log.warning("%s is already mounted." % target)
            else:
                args = self.cmd_mount + [p, target]
                proc_mnt = Popen( self.cmd_mount + [p, target], stdout=PIPE, stderr=STDOUT, close_fds=True )
                exception_if_error(proc_mnt, args)

    def removedirs(self, target):
        target = os.path.realpath(target)
        if target == self.root:
            return
        try:
            #os.removedirs(target)
            os.rmdir(target)
        except:
            return
        self.removedirs(os.path.dirname(target))

    def umount(self):
        self.assert_no_lock("commit")

        output = Popen([ "mount" ], stdout=PIPE, stderr=STDOUT, close_fds=True ).communicate()[0]
        pattern = re.compile(r"^(.*) on (.*) type .*$")
        mount_root = os.path.realpath( os.path.join( self.root, self.WORK_TREE) )
        mount_list = []
        for line in output.splitlines():
            m = pattern.search(line)
            if m:
                src = m.group(1)
                target = os.path.realpath( m.group(2) )
                if target.startswith( mount_root ):
                    mount_list.append( (target, src, ) )

        for target, src in sorted( mount_list, reverse=True ):
            try:
                args = self.cmd_umount + [ target ]
                proc_umnt = Popen( args, stdout=PIPE, stderr=STDOUT, close_fds=True )
                exception_if_error( proc_umnt, args, lambda n: n.endswith("not mounted") )
            except:
                args = self.cmd_umount_force + [ target ]
                proc_umnt = Popen( args, stdout=PIPE, stderr=STDOUT, close_fds=True )

                args = self.cmd_umount_force + [ target ]
                proc_umnt = Popen(args, stdout=PIPE, stderr=STDOUT, close_fds=True )
                exception_if_error( proc_umnt, args, lambda n: n.endswith("not mounted") )

        for target, src in sorted( mount_list, reverse=True ):
            log.debug("remove %s" % target)
            if not self.is_mount(src, target) and target.startswith( mount_root ) and target != mount_root :
                if os.path.isdir(target):
                    self.removedirs(target)
                else:
                    os.unlink(target)
                    target = os.path.dirname( target )
                    if target.startswith( mount_root ) and target != mount_root :
                        self.removedirs( target )

        # remote WORK_TREE dirs
        if os.path.exists( os.path.join( self.WORK_TREE, ".gitignore" ) ):
            os.unlink( os.path.join( self.WORK_TREE, ".gitignore" ) )
        runtimedir = self.WORK_TREE
        while runtimedir.startswith("/var/run/gistore/"):
            try:
                if os.path.exists( runtimedir ):
                    os.rmdir( runtimedir )
            except:
                log.error("Can not remove runtime-dir: %s" % runtimedir)
                break
            runtimedir = os.path.dirname( runtimedir )

        self.unlock("mount")


    def cleanup(self):
        self.unlock("commit")
        self.umount()

    def commit(self, message):
        self.assert_lock("mount")
        self.lock("commit")
        self.scm.commit( message )
        self.unlock("commit")

    def post_check(self):
        self.scm.post_check()

    def has_lock(self, event):
        lockfile = os.path.join( self.root, GISTORE_LOCK_DIR, "_gistore-lock-" + event )
        if os.path.exists( lockfile ):
            return True
        else:
            return False

    def lock(self, event):
        self.assert_no_lock(event)

        lockfile = os.path.join( self.root, GISTORE_LOCK_DIR, "_gistore-lock-" + event )
        f = open( lockfile, 'w' )
        f.write( str( os.getpid() ) )
        f.close()

    def assert_lock(self, event):
        lockfile = os.path.join( self.root, GISTORE_LOCK_DIR, "_gistore-lock-" + event )
        if not os.path.exists( lockfile ):
            raise GistoreLockError( "Has not lock using: %s" % lockfile )

    def assert_no_lock(self, event):
        lockfile = os.path.join( self.root, GISTORE_LOCK_DIR, "_gistore-lock-" + event )
        if os.path.exists( lockfile ):
            raise GistoreLockError( "Lock already exists: %s" % lockfile )

    def unlock(self, event):
        lockfile = os.path.join( self.root, GISTORE_LOCK_DIR, "_gistore-lock-" + event )
        try:
            os.unlink( lockfile )
        except OSError:
            pass

# vim: et ts=4 sw=4



# -*- coding: utf-8 -*-
#
# gistore -- Backup files using DVCS, such as git.
# Copyright (C) 2010 Jiang Xin <jiangxin@ossxp.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#


# -*- coding: utf-8 -*-
#
# gistore -- Backup files using DVCS, such as git.
# Copyright (C) 2010 Jiang Xin <jiangxin@ossxp.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

import sys
import os
import logging

GISTORE_CONFIG_DIR  = ".gistore"
GISTORE_LOG_DIR     = "logs"
GISTORE_LOCK_DIR    = "locks"

class DefaultConfig(object):
    sys_config_dir = os.environ.get('GISTORE_ETC') or '/etc/gistore'
    backend = "git"
    root_only = False
    log_level = logging.INFO
    task = None
    backup_history = 200
    backup_copies  = 5
    store_list = {'default': None }

def initConfig():
    task_dir = os.path.join(DefaultConfig.sys_config_dir, 'tasks')
    if not os.path.exists(task_dir):
        try:
            os.makedirs(task_dir)
        except OSError:
            print >> sys.stderr, "no permisson to create dir: %s" % task_dir
            return

    config_file = os.path.join(DefaultConfig.sys_config_dir, 'local_config.py')
    if os.path.exists(config_file):
        return

    try:
        fp = open(config_file, "w")
    except IOError:
        print >> sys.stderr, "no permisson to create file: %s" % config_file
    else:
        fp.write("""# -*- coding: utf-8 -*-
#
# gistore -- Backup files using DVCS, such as git.
# Copyright (C) 2010 Jiang Xin <jiangxin@ossxp.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

from gistore.config import DefaultConfig

DEFAULT_BACKUPS = []

##Example: dynamic backup list
#import subprocess
#proc = subprocess.Popen(
#           ['python', '/opt/ossxp/bin/ossxp_pkgadmin.py', 'list_backup'],
#           stdout=subprocess.PIPE, stderr=open('/dev/null', 'w'),
#           close_fds=True )
#for line in proc.communicate()[0].splitlines():
#    DEFAULT_BACKUPS.append(line.strip())

class Config(DefaultConfig):
    store_list = {'default': DEFAULT_BACKUPS }

# vim: et ts=4 sw=4
""")
        fp.close()

def getConfig():
    try:
        #Initial sys_config_dir if not exists.
        if not os.path.exists(os.path.join(DefaultConfig.sys_config_dir, 'local_config.py')):
            initConfig()

        # load custom config file in /etc/gistore/local_config.py
        sys.path.insert(0, DefaultConfig.sys_config_dir);
        module = __import__('local_config', globals(), {})
        configClass = getattr(module, 'Config')
        cfg = configClass()
    except ImportError:
        cfg = DefaultConfig()

    return cfg

cfg = getConfig()

# vim: et ts=4 sw=4



# -*- coding: utf-8 -*-
#
# gistore -- Backup files using DVCS, such as git.
# Copyright (C) 2010 Jiang Xin <jiangxin@ossxp.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

import os
import re
import subprocess
import logging
import socket

from gistore.scm.abstract import AbstractSCM
from gistore.utils import *
from gistore.config import *

log = logging.getLogger('gist.git')

class SCM(AbstractSCM):


    def __init__(self, root="", work_tree="run-time", backup_history=0, backup_copies=0):
        super(SCM, self).__init__(root, work_tree, backup_history, backup_copies)
        os.putenv("GIT_COMMITTER_NAME", self.username)
        os.putenv("GIT_COMMITTER_EMAIL", self.username+"@"+socket.gethostname())
        self.GIT_DIR     = "repo.git"

    def get_command(self, git_dir=True, work_tree=True):
        args = [ "git" ]
        if git_dir:
            args.append( "--git-dir=%s" % os.path.join(self.root, self.GIT_DIR) )
        if work_tree:
            args.append( "--work-tree=%s" % os.path.join(self.root, self.WORK_TREE) )

        return args

    command = property(get_command)


    def is_repos(self):
        return os.path.exists( os.path.join(self.root, self.GIT_DIR, 'objects') )


    def init(self):
        if self.is_repos():
            log.warning("Repos %s already exists." % self.root)
            return False

        commands = [ 
                    # git init command can not work with --work-tree arguments.
                    [ "git", "init", "--bare", os.path.join(self.root, self.GIT_DIR) ],

                    # a empty commit is used as root commit of rotate backup
                    self.get_command(work_tree=False) + [ "--work-tree=/tmp", "commit", "--allow-empty", "-m", "gistore root commit initialized." ],

                    # tag the empty commit as gistore/0, never delete it.
                    self.get_command(work_tree=False) + [ "tag", "gistore/0" ],

                    # set local git config, which not affect by global config
                    self.get_command(work_tree=False) + [ "config", "core.autocrlf", "false" ],
                    self.get_command(work_tree=False) + [ "config", "core.safecrlf", "false" ],
                    self.get_command(work_tree=False) + [ "config", "core.symlinks", "true" ],
                    self.get_command(work_tree=False) + [ "config", "core.trustctime", "false" ],
                    self.get_command(work_tree=False) + [ "config", "core.sharedRepository", "group" ],

                    # in case of merge, use ours instead.
                    self.get_command(work_tree=False) + [ "config", "merge.ours.name", "\"always keep ours\" merge driver" ],
                    self.get_command(work_tree=False) + [ "config", "merge.ours.driver", "touch %A" ],
                   ]

        for args in commands:
            proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            exception_if_error(proc, args)

        #log.debug("create .gitignore")
        #fp = open(os.path.join(self.root, self.WORK_TREE, ".gitignore"), "w")
        #fp.write(".gistore-*\n")
        #fp.close()


    def upgrade(self, old):
        if old < 2:
            self.upgrade_2()


    def upgrade_2(self):
        if self.GIT_DIR != ".git":
            oldgit = os.path.join(self.root, ".git")
            if os.path.exists( oldgit ):
                os.rename( oldgit, os.path.join(self.root, self.GIT_DIR) )

        commands = []
        args = self.get_command(work_tree=False) + [ "show-ref" ]
        returncode = subprocess.call( args=args, stdout=sys.stderr, close_fds=True )
        # repo.git has no commit yet
        if returncode != 0:
            commands += [
                          # a empty commit is used as root commit of rotate backup
                          self.get_command(work_tree=False) + [ "--work-tree=/tmp", "commit", "--allow-empty", "-m", "gistore root commit initialized." ],

                          # tag the empty commit as gistore/0, never delete it.
                          self.get_command(work_tree=False) + [ "tag", "gistore/0" ],
                         ]
        else:
            commands += [
                          # switch to a unexist ref
                          self.get_command(work_tree=False) + [ "symbolic-ref", "HEAD", "refs/tags/gistore/0" ],

                          # remove cached index
                          self.get_command(work_tree=False) + [ "rm", "--cached", "-r", "-f", "-q", "." ],

                          # a empty commit is used as root commit of rotate backup
                          self.get_command(work_tree=False) + [ "--work-tree=/tmp", "commit", "--allow-empty", "-m", "gistore root commit initialized." ],

                          # switch to master
                          self.get_command(work_tree=False) + [ "symbolic-ref", "HEAD", "refs/heads/master" ],
                          self.get_command(work_tree=False) + [ "--work-tree=/tmp", "reset", "HEAD" ],
                         ]

        commands +=[ 
                    # set as bare repos
                    self.get_command(work_tree=False) + [ "config", "core.bare", "true" ],

                    # set local git config, which not affect by global config
                    self.get_command(work_tree=False) + [ "config", "core.autocrlf", "false" ],
                    self.get_command(work_tree=False) + [ "config", "core.safecrlf", "false" ],
                    self.get_command(work_tree=False) + [ "config", "core.symlinks", "true" ],
                    self.get_command(work_tree=False) + [ "config", "core.trustctime", "false" ],
                    self.get_command(work_tree=False) + [ "config", "core.sharedRepository", "group" ],

                    # in case of merge, use ours instead.
                    self.get_command(work_tree=False) + [ "config", "merge.ours.name", "\"always keep ours\" merge driver" ],
                    self.get_command(work_tree=False) + [ "config", "merge.ours.driver", "touch %A" ],
                   ]

        for args in commands:
            proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            exception_if_error(proc, args)


    def _get_commit_count(self):
        args = self.get_command(work_tree=False) + [ "rev-list", "master" ]
        log.debug(" ".join(args))
        proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=None, close_fds=True)
        lines = proc.communicate()[0].splitlines()
        if proc.returncode != 0:
            msg = "Last command: %s\n\tgenerate ERRORS with returncode %d!" % (cmdline, proc.returncode)
            log.critical( msg )
            if lines:
                log.critical( "Command output:\n" + ''.join(lines))
            raise CommandError( msg )

        log.debug( "Total %d commits in master." % len(lines) )

        # the very first commit is a blank commit, nothing in it.
        return len(lines) - 1


    def backup_rotate(self):
        if not self.backup_history or self.backup_history < 1:
            return
        if not self.backup_copies or self.backup_copies < 1:
            return

        count = self._get_commit_count()
        # the first commit is a blank commit
        if count < self.backup_history:
            log.debug( "No backup rotate needed. %d < %d." % (count, self.backup_history) )
            return

        log.info( "Begin backup rotate, for %d >= %d." % (count, self.backup_history) )
        # list branches with prefix: gistore/
        args = self.get_command(work_tree=False) + [ "branch" ]
        proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=None, close_fds=True)
        lines = sorted( proc.communicate()[0].splitlines() )
        if proc.returncode != 0:
            msg = "Last command: %s\n\tgenerate ERRORS with returncode %d!" % (cmdline, proc.returncode)
            log.critical( msg )
            if lines:
                log.critical( "Command output:\n" + ''.join(lines) )
            raise CommandError( msg )
        tagids = []
        for tag in lines:
            tag=tag.strip()
            if not tag.startswith("gistore/"):
                continue
            try:
                tagN = int(tag[8:])
            except ValueError:
                continue
            if tagN == 0:
                continue
            tagids.append( tagN )

        command_list = []
        tagids = sorted( tagids )

        # rotate tags, and add new tag
        if len(tagids) >= self.backup_copies:
            for i in range(1, self.backup_copies):
                cmd = self.get_command(work_tree=False) + [ "update-ref", "refs/heads/gistore/%d" % i, "refs/heads/gistore/%d" % tagids[ i - self.backup_copies ] ]
                command_list.append(cmd)
            for i in tagids:
                if i in range(1, self.backup_copies):
                    continue
                cmd = self.get_command(work_tree=False) + [ "branch", "-D", "gistore/%d" % i ]
                command_list.append(cmd)
            cmd = self.get_command(work_tree=False) + [ "branch", "gistore/%d" % self.backup_copies, "master" ]
            command_list.append(cmd)
        else:
            if len(tagids) > 0:
                cmd = self.get_command(work_tree=False) + [ "branch", "gistore/%d" % (tagids[-1] + 1), "master" ]
            else:
                cmd = self.get_command(work_tree=False) + [ "branch", "gistore/1", "master" ]
            command_list.append(cmd)

        # reset master to gistore/0
        cmd = self.get_command(work_tree=False) + [ "update-ref", "refs/heads/master", "refs/tags/gistore/0" ]
        command_list.append(cmd)
        # reset HEAD
        cmd = self.get_command() + [ "reset", "HEAD" ]
        command_list.append(cmd)
        # do gc
        cmd = self.get_command(work_tree=False) + [ "gc" ]
        command_list.append(cmd)

        for i in range(len(command_list)):
            log.debug( "Backup rotate step %d" % i )
            proc = subprocess.Popen(command_list[i], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            exception_if_error(proc, command_list[i])


    def commit(self, message=None):

        # generate user friendly commit log
        def commit_summary( commit_st ):
            sample = 5
            status = {}
            buffer = []
            for line in commit_st:
                k,v = line.split(None, 1)
                try:
                    status[k].append( v )
                except KeyError:
                    status[k] = [ v ]

            total = len( commit_st )
            buffer.append( "Changes summary: total= %(total)d, %(detail)s" % {
                    'total': total,
                    'detail': ", ".join( [ "%s: %d" % (k, len(status[k])) for k in sorted(status.keys()) ] ),
                    } )
            buffer.append( "-" * len(buffer[0]) )
            for k in sorted(status.keys()):
                brief_st = []
                total = len( status[k] )
                step = total // sample
                if step < 1:
                    brief_st = status[k]
                else:
                    for i in range(sample):
                        brief_st.append( status[k][i*step] )
                    brief_st.append( "...%d more..." % ( len(status[k]) - sample ) )
                buffer.append( "    %s => %s" % ( k, ", ".join(brief_st) ) )

            return "\n".join(buffer)


        # add submodule as normal directory
        def add_submodule(submodule, status=[]):
            # submodule already deleted in cache

            # add tmp file in submodule
            open( os.path.join( self.root, self.WORK_TREE, submodule, '.gistore-submodule'), 'w' ).close()

            # git add tmp file in submodule
            args = self.command + [ "add", "-f", os.path.join( submodule, '.gistore-submodule' ) ]
            proc_add = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            exception_if_error(proc_add, args)

            # git add whole submodule dir
            args = self.command + [ "add", submodule ]
            proc_add = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            exception_if_error(proc_add, args)

            # git rm -f tmp file in submodule
            args = self.command + [ "rm", "-f", os.path.join( submodule, '.gistore-submodule' ) ]
            proc_rm = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            exception_if_error(proc_rm, args)

            # check status --porcelain and append to status[]
            args = self.command + [ "status", "--porcelain", submodule ]
            log.debug(" ".join(args))
            proc_st = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            status.extend( [ line for line in proc_st.communicate()[0].splitlines() ] )
            return status


        ##
        self._abort_if_not_repos()

        # Check if backup needs rotate
        self.backup_rotate()

        args = self.command + [ "add", "." ]
        proc_add = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
        exception_if_error(proc_add, args)

        # delete files but keep directories.
        if True:
            args = self.command + [ "ls-files", "--deleted" ]
            log.debug(" ".join(args))
            proc_ls = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            deleted_files = []
            for file in proc_ls.communicate()[0].splitlines():
                deleted_files.append(file)
            if deleted_files:
                try:
                    # `git rm --cached` will not remote blank-dir.
                    args = self.get_command(work_tree=False) + [ "rm", "--cached", "--quiet" ] + deleted_files
                    proc_rm = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
                    warn_if_error(proc_rm, args)
                except OSError, e:
                    if "Argument list too long" in e:
                        for file in deleted_files:
                            args = self.get_command(work_tree=False) + [ "rm", "--cached", "--quiet", file ]
                            proc_rm = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
                            warn_if_error(proc_rm, args)

        args = self.command + [ "status", "--porcelain" ]
        log.debug(" ".join(args))
        proc_st = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
        commit_stat = [ line for line in proc_st.communicate()[0].splitlines() ]

        submodules = self.remove_submodules()
        while submodules:
            log.info( "Re-add submodules: %s", ', '.join(submodules) )
            for submod in submodules:
                commit_stat.extend( add_submodule(submod) )
            # new add directories may contain other submodule.
            submodules = self.remove_submodules()

        if message:
            message += "\n\n"
            message += commit_summary( commit_stat )
        else:
            message = commit_summary( commit_stat )

        if commit_stat:
            log.info( "Backup changes for %s\n%s" % (self.root, message) )
        else:
            log.info( "*Nothing changed*, no backup for %s\n%s" % (self.root, message) )

        msgfile = os.path.join( self.root, GISTORE_LOG_DIR, "COMMIT_MSG" )
        fp = open( msgfile, "w" )
        fp.write( message )
        fp.close()

        args = self.command + [ "commit", "--quiet", "-F", msgfile ]
        proc_ci = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
        # If nothing to commit, git commit return 1.
        exception_if_error( proc_ci, args, lambda n: n.startswith("nothing to commit") or n.startswith("no changes added to commit") )


    def remove_submodules(self):
        submodules = []
        args = self.get_command(work_tree=False) + [ "--work-tree=.", "submodule", "status" ]
        proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
        pat1 = re.compile(r".\w{40} (\w*) \(.*\)?")
        pat2 = re.compile(r"No submodule mapping found in .gitmodules for path '(.*)'")
        for line in proc.communicate()[0].splitlines():
            line = line.strip()
            m = pat1.match(line)
            if m:
                submodules.append(m.group(1))
            m = pat2.match(line)
            if m:
                submodules.append(m.group(1))

        if submodules:
            log.warning("Remove submodules in backup:"+"\n    "+" ".join(submodules))
            args = self.get_command(work_tree=False) + [ "rm", "--cached", "-q" ] + submodules
            proc_rm = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            exception_if_error(proc_rm, args)

            # maybe other submodules
            submodules.extend( self.remove_submodules() )
            return submodules

        else:
            return []


# vim: et ts=4 sw=4



# -*- coding: utf-8 -*-
#
# gistore -- Backup files using DVCS, such as git.
# Copyright (C) 2010 Jiang Xin <jiangxin@ossxp.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

'''PROGRAM INTRODUCTION
Gistore is a backup system with a DVCS as backend, and git is highly
recommended. Orignally or currently only git is supportted. So, you
must have git installed. For git installation, see http://git-scm.com/.

Gistore can *store* files and directories from any place, without copying
or sync files to the gistore DVCS working tree. All the backup files
and directories are mounted to the gistore DVCS working tree on the fly.

Usage: %(PROGRAM)s [options]

Options:

    -h|--help
        Print this message and exit.
    -v|--verbose
        Verbose mode: more debug message
    -q|--quiet
        Quiet mode: less message
    -r <taskname> | -r <repos_dir>
        Set taskname or repos directory

Available command:
    list
        List tasks linked to /etc/gistore/tasks/

    status [task or direcotry]
        Show backup repository's backup list and other status

    init   [task or directory]
        Initial gistore backup repository

    commit [task or direcotry]
        Commit changes in backup repository. The following command run 
        in order:
          * mount
          * commit
          * umount

    commit-all
        Commit changes in all backup tasks under /etc/gistore/tasks/
'''

import os
import sys
import getopt
import signal
import logging

if __name__ == '__main__':
    sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))

from gistore.utils  import *
from gistore.config import *
from gistore.errors import *
from gistore.api    import Gistore
from gistore.versions import *

log = logging.getLogger('gist.main')

def get_log_level( level ):
    try:
        level = int( level )
    except ValueError:
        level = logging._levelNames.get ( level.upper(), logging.DEBUG )

    if level <= 9:
        if level >= 5:
            level = logging.DEBUG
        elif level >= 4:
            level = logging.INFO
        elif level >= 3:
            level = logging.WARNING
        elif level >= 2:
            level = logging.ERROR
        elif level <= 1:
            level = logging.CRITICAL

    # level is number and >= 10
    elif not logging._levelNames.has_key( level ):
        level = logging.DEBUG

    return level


class GistoreCmd(object):
    opt_verbose = 4     # info
    opt_message = None

    gistobj = None

    @staticmethod
    def do_mount(args=[]):
        if not args:
            args = [None]
        for repos in args:
            try:
                GistoreCmd.gistobj = Gistore(repos)
                GistoreCmd.gistobj.mount()
            except GistoreLockError, e:
                logging.critical( get_exception(e) )
                continue

    @staticmethod
    def do_init(args=[]):
        if not args:
            args = [None]
        for repos in args:
            try:
                GistoreCmd.gistobj = Gistore(repos, True)
                GistoreCmd.gistobj.init()
            except GistoreLockError, e:
                logging.critical( get_exception(e) )
                continue
            except Exception, e:
                logging.critical( get_exception(e) )
                # remove lock files...
                GistoreCmd.cleanup()
                continue


    @staticmethod
    def do_list(args=[]):
        if not args:
            print "Task list:"
            tasksdir = os.path.join(cfg.sys_config_dir, 'tasks')
            if os.path.exists(tasksdir):
                for t in sorted(os.listdir(tasksdir)):
                    if t.startswith("."):
                        continue
                    dest = os.path.realpath(os.path.join(tasksdir,t))
                    print "    %-10s: %s" % (t, dest)
        else:
            GistoreCmd.do_status(args)


    @staticmethod
    def do_status(args=[]):
        if not args:
            args = [None]
        for repos in args:
            try:
                GistoreCmd.gistobj = Gistore(repos)
                GistoreCmd.gistobj.status()
                GistoreCmd.gistobj.post_check()
            except Exception, e:
                logging.critical( get_exception(e) )
                continue


    @staticmethod
    def do_umount(args=[]):
        if not args:
            args = [None]
        for repos in args:
            try:
                GistoreCmd.gistobj = Gistore(repos)
                GistoreCmd.gistobj.umount()
            except GistoreLockError, e:
                logging.critical( get_exception(e) )
                continue
            except Exception, e:
                logging.critical( get_exception(e) )
                GistoreCmd.cleanup()
                continue

    @staticmethod
    def do_forbidden(args=[]):
        raise Exception("Internal use only, not run by user.")

    @staticmethod
    def do_commit_all(args=[]):
        tasks = []
        tasksdir = os.path.join(cfg.sys_config_dir, 'tasks')
        if os.path.exists(tasksdir):
            for t in sorted(os.listdir(tasksdir)):
                if t.startswith("."):
                    continue
                tasks.append(os.path.realpath(os.path.join(tasksdir,t)))
        if tasks:
            GistoreCmd.do_commit(tasks)


    @staticmethod
    def do_commit(args=[]):
        if not args:
            args = [None]
        for repos in args:
            try:
                GistoreCmd.gistobj = Gistore(repos)
                GistoreCmd.gistobj.mount()
                GistoreCmd.gistobj.commit( GistoreCmd.opt_message )
                GistoreCmd.gistobj.umount()
                GistoreCmd.gistobj.post_check()
            except GistoreLockError, e:
                logging.critical( get_exception(e) )
                continue
            except TaskNotExistsError, e:
                logging.critical( get_exception(e) )
                continue
            except Exception, e:
                logging.critical( get_exception(e) )
                # remove lock files...
                GistoreCmd.cleanup()
                continue


    @staticmethod
    def sigint_handler(signum, frame):
        """Do umount and others cleanups if receive SIGINT.
        """
        log.critical("Catch SIGINT. Cleanup then exit...")
        GistoreCmd.cleanup()
        signal.signal(signal.SIGINT, signal.default_int_handler)
        sys.exit(1)

    @staticmethod
    def cleanup():
        if GistoreCmd.gistobj is not None:
            log.warning("Doing cleanups...")
            GistoreCmd.gistobj.cleanup()
        else:
            log.debug("Not doing cleanups...")

    @staticmethod
    def usage(code, msg=''):
        if code:
            fd = sys.stderr
        else:
            fd = sys.stdout
        print >> fd, __doc__ % { 'PROGRAM':os.path.basename(sys.argv[0]) }
        if msg:
            print >> fd, msg
        sys.exit(code)

    @staticmethod
    def parse_options(argv):
        global cfg
        try:
            opts, args = getopt.getopt(
                argv, "hvqm:",
                [ "help", "verbose", "quiet" ])
        except getopt.error, msg:
            return GistoreCmd.usage(1, msg)

        for opt, arg in opts:
            if opt in ('-h', '--help'):
                return GistoreCmd.usage(0)
            elif opt in ('-v', '--verbose'):
                if GistoreCmd.opt_verbose >= 5:
                    GistoreCmd.opt_verbose = 5
                else:
                    GistoreCmd.opt_verbose += 1
            elif opt in ('-q', '--quiet'):
                if GistoreCmd.opt_verbose <= 0:
                    GistoreCmd.opt_verbose = 0
                else:
                    GistoreCmd.opt_verbose -= 1
            elif opt in ('-m'):
                GistoreCmd.opt_message = arg

        if GistoreCmd.opt_verbose is not None:
            cfg.log_level = GistoreCmd.opt_verbose
        cfg.log_level = get_log_level(cfg.log_level)

        return args

    @staticmethod
    def main(argv=None):
        # Parse global options...
        if argv is None:
            argv = sys.argv[1:]
        args = GistoreCmd.parse_options(argv)
        if not args:
            return GistoreCmd.usage(0)

        command = args[0].lower().replace('-','_')
        # Command aliases
        if command in ['umount', 'unmount', 'umnt', 'unmnt']:
            command = 'forbidden'
        elif command in ['mount', 'mnt']:
            command = 'forbidden'
        elif "initialized".startswith(command):
            command = 'init'
        elif command in ["ci", "backup"]:
            command = 'commit'
        elif command in ["ci_all", "backup_all"]:
            command = 'commit_all'
        elif "status".startswith(command):
            command = 'status'
        elif command in ["ls", "dir"]:
            command = 'list'

        # Parse command options, and args is command params.
        args = GistoreCmd.parse_options(args[1:])

        # Set logging basic config, to stderr
        logging.basicConfig( level=cfg.log_level,
                format='%(asctime)s %(levelname)-8s %(message)s',
                datefmt='%m-%d %T')

        try:
            # Override SIGINT handler, do umount and other cleanups...
            signal.signal(signal.SIGINT,  GistoreCmd.sigint_handler)
            signal.signal(signal.SIGHUP,  GistoreCmd.sigint_handler)
            signal.signal(signal.SIGQUIT, GistoreCmd.sigint_handler)
            signal.signal(signal.SIGABRT, GistoreCmd.sigint_handler)
            signal.signal(signal.SIGSEGV, GistoreCmd.sigint_handler)
            signal.signal(signal.SIGPIPE, GistoreCmd.sigint_handler)
            signal.signal(signal.SIGTERM, GistoreCmd.sigint_handler)

            if 'do_'+command in filter(lambda n: n.startswith('do_'), dir(GistoreCmd)):
                getattr(GistoreCmd, 'do_' + command)(args)
            else:
                return GistoreCmd.usage(1, "Unknown command: %s" % command)
        except GistoreLockError, e:
            logging.critical( str(e) )
            raise e
        except Exception, e:
            logging.critical( get_exception(e) )
            GistoreCmd.sigint_handler

def main(argv=None):
    GistoreCmd.main(argv)

if __name__ == '__main__':
	sys.exit(main())

# vim: et ts=4 sw=4



#!/usr/bin/env python

from distutils.core import setup

version_string = "0.2.5"

setup(name='gistore',
    version=version_string,
    description='Backup system using DVCS backend, such as git',
    author='Jiang Xin',
    author_email='jiangxin@ossxp.com',
    url='http://open.ossxp.com/',
    package_dir = {'': 'lib'},
    packages=['gistore','gistore.scm'],
    scripts = ['gistore'],
    data_files = [('share/doc/gistore', ['CHANGELOG', 'COPYING', 'README']), ]
    )

# vim: et sw=4 ts=4



# -*- coding: utf-8 -*-
#
# gistore -- Backup files using DVCS, such as git.
# Copyright (C) 2010 Jiang Xin <jiangxin@ossxp.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

import os
import pwd
from gistore.errors import *

class AbstractSCM(object):


    def __init__(self, root="", work_tree=".", backup_history=0, backup_copies=0):
        self.set_root(root)
        self.backup_history = backup_history
        self.backup_copies  = backup_copies
        self.username = pwd.getpwuid(os.getuid())[0] or "gistore"
        self.WORK_TREE = work_tree

    def set_root(self, root=""):
        if not root:
            root = os.getcwd()
        self.root = os.path.realpath(root)
        
    def _abort_if_not_repos(self):
        if not self.is_repos():
            raise UninitializedRepositoryError

    def is_repos(self):
        raise NoImplementError("not implement is_repos")

    def init(self):
        raise NoImplementError("not implement init")

    def commit(self):
        raise NoImplementError("not implement commit")

    def post_check(self):
        return

